<?php
/**
 * @file
 * Fivestar component for webform module. Implements webform component hooks.
 *
 */

/**
 * Implementation of _webform_defaults_component().
 */
function _webform_defaults_fivestar() {
  return array(
    'name' => '',
    'form_key' => NULL,
    'pid' => 0,
    'weight' => 0,
    'value' => '5',
    'mandatory' => 0,
    'email' => 1,
    'extra' => array(
      'admin_display' => 'stars',
      'field_prefix' => '',
      'field_suffix' => '',
      'disabled' => 0,
      'unique' => 0,
      'description' => '',
      'attributes' => array(),
      'default_value' => 0,
      'count_zero_ratings' => 0,
      'admin_analysis' => 'full'
    ),
  );
}




/**
 * Implementation of _webform_edit_component().
 */
function _webform_edit_fivestar($component) {
  $form = array();

  // Disabling the description if not wanted.
  $form['description'] = array();

  $form['value'] = array(
    '#type' => 'radios',
    '#title' => t('Number of stars'),
    '#default_value' => $component['value'],
    '#description' => t('The number of stars to display'),
    '#weight' => 5,
    '#required' => TRUE,
    '#options' => array(
      '2' => t('Two stars'),
      '3' => t('Three stars'),
      '4' => t('Four stars'),
      '5' => t('Five stars'),
      '6' => t('Six stars'),
      '7' => t('Seven stars'),
      '8' => t('Eight stars'),
      '9' => t('Nine stars'),
      '10' => t('Ten stars'),
    ),
  );

  $form['extra']['count_zero_ratings'] = array(
    '#type' => 'radios',
    '#title' => t('Count zero ratings'),
    '#default_value' => $component['extra']['count_zero_ratings'],
    '#description' => t('Use zero ratings in calculations of averages. When working with a non-mandatory field then it may be useful not to count zeros.'),
    '#weight' => 6,
    '#required' => TRUE,
    '#options' => array(
      '0' =>  t('No'),
      '1' =>  t('Yes'),
    ),
  );

  $form['extra']['default_value'] = array(
    '#type' => 'radios',
    '#title' => t('Default number of stars'),
    '#default_value' => $component['extra']['default_value'],
    '#description' => t('The default number of stars to mark when the user first looks at the form.'),
    '#weight' => 5,
    '#required' => TRUE,
    '#options' => array(
      '0' =>   t('None'),
      '50' =>  t('Half'),
      '100' => t('All'),
    ),
  );

  $form['display']['admin_display'] = array(
    '#type' => 'radios',
    '#title' => t('Administration display settings'),
    '#default_value' => isset($component['extra']['admin_display']) ? $component['extra']['admin_display'] : 'stars',
    '#description' => t('Chooses between displaying results in absolute or starred values.'),
    '#weight' => 5,
    '#options' => array(
      'stars' => t('Use stars in the results display.'),
      'percentage' => t('Use numbers in the results display.'),
    ),
    '#parents' => array('extra', 'admin_display'),
  );

  $form['display']['admin_analysis'] = array(
    '#type' => 'radios',
    '#title' => t('Administration analysis display settings'),
    '#default_value' => isset($component['extra']['admin_analysis']) ? $component['extra']['admin_analysis'] : 'full',
    '#description' => t('Choose how to report results on the analysis page.'),
    '#weight' => 5,
    '#options' => array(
      'basic' => t('Only show a percentage.'),
      'full' => t('Show a full breakdown of the ratings.'),
  ),
    '#parents' => array('extra', 'admin_analysis'),
  );

  return $form;
}

/**
 * Implementation of _webform_render_component().
 */
function _webform_render_fivestar($component, $value = NULL, $filter = TRUE) {

  $element = array(
    '#type'          => 'fivestar',
    '#title'         => $component['name'],
    '#stars'         => $component['value'],
    '#default_value' => $component['extra']['default_value'],
    '#required'      => $component['mandatory'],
    '#weight'        => $component['weight'],
    '#description'   => $component['extra']['description'],
    '#attributes'    => $component['extra']['attributes'],
    '#prefix'        => '<div class="webform-component webform-component-' . $component['type'] . '" id="webform-component-' . $component['form_key'] . '">',
    '#suffix'        => '</div>',
    '#webform_component' => $component,
    '#element_validate' => array(),
  );

  // If resuming the form, put in the current value
  if (isset($value)) {
    $form_item['#default_value'] = $value[0];
  }

  return $element;
}


function _webform_display_fivestar($component, $value, $format = 'html') {
 /*
  *  Component holds values for this instance of fivestar.
  *  $component['value'] hold the number of stars to show
  */
  if ($format != 'html') {
    $star_weight = (100/$component['value']);
    $stars = ceil($value[0] / $star_weight);
    $output = $stars . '/' . $component['value'] . ' ' . t('stars');
  }
  else {
    $output = _webform_fivestar_render_score($component, $value[0]);
  }

  return array(
    '#title' => $component['name'],
    '#weight' => $component['weight'],
    '#theme_wrappers' => $format == 'html' ? array('webform_element') : array('webform_element_text'),
    '#post_render' => array('webform_element_wrapper'),
    '#format' => $format,
    '#value' => $output,
  );
}

/**
 * Calculate and returns statistics about results for this component.
 *
 * This takes into account all submissions to this webform. The output of this
 * function will be displayed under the "Results" tab then "Analysis".
 *
 * @param $component
 *   An array of information describing the component, directly correlating to
 *   the webform_component database schema.
 * @param $sids
 *   An optional array of submission IDs (sid). If supplied, the analysis will
 *   be limited to these sids.
 * @param $single
 *   Boolean flag determining if the details about a single component are being
 *   shown. May be used to provided detailed information about a single
 *   component's analysis, such as showing "Other" options within a select list.
 * @return
 *   An array of data rows, each containing a statistic for this component's
 *   submissions.
 */
function _webform_analysis_fivestar($component, $sids = array(), $single = FALSE) {
  $rows = array();
  $submission_count = 0;
  $rating_total = 0;
  $star_weight = ceil(100/$component['value']);
  $analysis = $component['extra']['admin_analysis'];

  $sidfilter = count($sids) ? " AND sid in (" . db_placeholders($sids, 'int') . ")" : "";
  $query = 'SELECT data '.
    ' FROM {webform_submitted_data} '.
    ' WHERE nid = %d '.
    ' AND cid = %d '.
    " AND data != '' ". $sidfilter ;
  $result = db_query($query, array_merge(array($component['nid'], $component['cid']), $sids));

  while ($data = db_fetch_object($result)) {
    $submission_count ++;
    $rating_total += $data->data;
    $stars = ceil($data->data / $star_weight);
    $ranges[$stars] ++;
  }

  if ($component['extra']['count_zero_ratings']) {
    //Calculating of average including zero ratings
    $average_rating = $submission_count ? round(($rating_total/$submission_count), 1) : 0;
    $rows[] = array( t('Average rating (using zero ratings)'), $average_rating . '%');
  }
  else { //Calculating of average without zero ratings
    $zeroonly_submissions = $submission_count == $ranges[0]; // Check for results where all submissions are zero. 
    $average_rating = (!$zeroonly_submissions && $submission_count) ? round($rating_total/($submission_count- $ranges[0]), 1) : 0;
    $rows[] = array( t('Average rating (without using zero ratings)'), $average_rating . '%');
  }

  if ($analysis == 'full') {
    // Special case of zero percent
    if (!empty($ranges[0])) {
      $rows[]= array( t('Number of 0 star reviews (0%)'), empty($ranges[0]) ? 0 : $ranges[0] );
    }
    
    for ($i = 1; $i <= $component['value']; $i++) {
      $ranges[$i] = empty($ranges[$i]) ? 0 : $ranges[$i];
      $lower_bound = round($star_weight*($i-1)) ;
      $upper_bound = round($star_weight*$i);
      $row_name = t('Number of !i star reviews (>!lower_bound% - !upper_bound%) ', array('!i' => $i, '!lower_bound' => $lower_bound, '!upper_bound' => $upper_bound ));
      $rows[] = array( $row_name, $ranges[$i]);
    }

    $row_name = '<strong>' . t('Total number of submissions') . '</strong>';
    $row_value = '<strong>' . $submission_count . '</strong>';
    $rows[] = array($row_name , $row_value);
  }
  return $rows;
}



/**
 * Return the result of a component value for display in a table.
 *
 * The output of this function will be displayed under the "Results" tab then
 * "Table".
 *
 * @param $component
 *   A Webform component array.
 * @param $value
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database schema.
 * @return
 *   Textual output formatted for human reading.
 */
function _webform_table_fivestar($component, $value) {
  return _webform_fivestar_render_score($component, $value[0]);
}


function _webform_fivestar_render_score($component, $score) {
  if (isset($component['extra']['admin_display'])) {
    if ($component['extra']['admin_display'] == 'stars') {
      $output = theme('fivestar_static', $score, $component['value']);
    }
    else {
      $output = $score . '%';
    }
  }
  return $output;
}




/**
 * Return the header for this component to be displayed in a CSV file.
 *
 * The output of this function will be displayed under the "Results" tab then
 * "Download".
 *
 * @param $component
 *   A Webform component array.
 * @param $export_options
 *   An array of options that may configure export of this field.
 * @return
 *   An array of data to be displayed in the first three rows of a CSV file, not
 *   including either prefixed or trailing commas.
 */
function _webform_csv_headers_fivestar($component, $export_options) {
  $header = array();
  $header[2] = array($component['name']);
  return $header;
}

/**
 * Format the submitted data of a component for CSV downloading.
 *
 * The output of this function will be displayed under the "Results" tab then
 * "Download".
 *
 * @param $component
 *   A Webform component array.
 * @param $export_options
 *   An array of options that may configure export of this field.
 * @param $value
 *   An array of information containing the submission result, directly
 *   correlating to the webform_submitted_data database schema.
 * @return
 *   An array of items to be added to the CSV file. Each value within the array
 *   will be another column within the file. This function is called once for
 *   every row of data.
 */
function _webform_csv_data_fivestar($component, $export_options, $value) {

  return $value;
}


